"use strict";

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function getResourceLoaderUrl() {
            return window.resourceLoaderUrl || "/explore/resourcePackageItem/";
        }
        function createJQueryResourceItemProxy(promiseFactory) {
            return new ResourceItemProxy(promiseFactory);
        }
        visualSandbox.getResourceLoaderUrl = getResourceLoaderUrl, visualSandbox.createJQueryResourceItemProxy = createJQueryResourceItemProxy;
        var ResourceItemProxy = function() {
            function ResourceItemProxy(promiseFactory) {
                this.promiseFactory = promiseFactory;
            }
            return ResourceItemProxy.prototype.get = function(resourcePackageId, name, path) {
                var deferred = this.promiseFactory.defer(), clusterUri = window.clusterUri, accessToken = window.powerBIAccessToken;
                top && top.powerBIAccessToken && (accessToken = top.powerBIAccessToken);
                var resourceLoaderUri = getResourceLoaderUrl(), headers = {
                    authorization: "Bearer " + accessToken
                };
                window.isAnonymousEmbed && (clusterUri = window.clusterApiUri, resourceLoaderUri = window.resourceLoaderUrl + resourcePackageId + "/", 
                headers["X-PowerBI-ResourceKey"] = window.powerBIResourceKey);
                var resourceToLoad = clusterUri + resourceLoaderUri + encodeURIComponent(path), ajaxSettings = {
                    url: resourceToLoad,
                    headers: headers
                };
                return $.ajax(ajaxSettings).then(function(data) {
                    return deferred.resolve(data);
                }, function(err) {
                    return deferred.reject(err);
                }), deferred.promise;
            }, ResourceItemProxy.prototype.getBlob = function(resourcePackageId, name, path) {
                return powerbi.createJQueryPromiseFactory().reject();
            }, ResourceItemProxy;
        }();
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createResourceLoaderHost() {
            var promiseFactory = powerbi.createJQueryPromiseFactory(), resourceProvider = {
                get: function(key) {
                    return key;
                },
                getOptional: function(key) {
                    return key;
                }
            }, services = {
                messageProxy: visualHost.createWindowMessageProxy(promiseFactory, window.parent),
                promiseFactory: promiseFactory,
                resourceProvider: resourceProvider
            };
            return new ResourceLoaderHost(services);
        }
        var visualHost = powerbi.visualHost, ResourcePackageItemType = powerbi.contracts.ResourcePackageItemType;
        visualSandbox.createResourceLoaderHost = createResourceLoaderHost;
        var ResourceLoaderHost = function() {
            function ResourceLoaderHost(services) {
                var _this = this;
                this.messageProxy = services.messageProxy, this.promiseFactory = services.promiseFactory, 
                this.resourceProvider = services.resourceProvider, this.messageProxy.on(visualHost.events.WindowMessageProxyMessageEvent, function(e) {
                    return _this.onMessageReceived(e);
                });
            }
            return ResourceLoaderHost.prototype.onMessageReceived = function(e) {
                switch (e.message.eventName) {
                  case "loadVisualPluginWithContent":
                    return this.loadVisualPluginWithContent.apply(this, e.message.arguments);
                }
            }, ResourceLoaderHost.prototype.loadVisualPluginWithContent = function(mappedPluginNameAndContent) {
                var deferred = this.promiseFactory.defer(), name = mappedPluginNameAndContent.pluginName, mappedPluginContent = mappedPluginNameAndContent.mappedPluginContent, jsContent = mappedPluginContent[ResourcePackageItemType.CustomVisualJavascript].content, cssContent = mappedPluginContent[ResourcePackageItemType.CustomVisualsCss].content;
                try {
                    visualHost.resourceInjector.injectJsCode(name, jsContent), visualHost.resourceInjector.injectCssCode(name, cssContent);
                    var plugin = powerbi.visuals.plugins[name];
                    plugin.iconUrl = visualHost.visualPluginServiceHelpers.getPluginIconUrl(name), plugin.content = {
                        js: jsContent,
                        css: cssContent,
                        iconBase64: plugin.iconUrl
                    }, powerbi.extensibility.visualPluginAdapter.transformPlugin(plugin), plugin.capabilities.objects = powerbi.data.services.DataViewObjectDescriptorSerializer.serialize(plugin.capabilities.objects, this.resourceProvider), 
                    deferred.resolve(plugin);
                } catch (err) {
                    deferred.reject(err);
                }
                return deferred.promise;
            }, ResourceLoaderHost;
        }();
        visualSandbox.ResourceLoaderHost = ResourceLoaderHost;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxHost(element, isolate) {
            void 0 === isolate && (isolate = !1);
            var promiseFactory = powerbi.createJQueryPromiseFactory(), messageProxy = powerbi.visualHost.createWindowMessageProxy(promiseFactory, window.parent), resourceProvider = {
                get: function(name) {
                    return name;
                },
                getOptional: function(name) {
                    return name;
                }
            };
            visualSandbox.SandboxVisualHostServices.initialize();
            var resourceLoaderConstructionOptions = {
                promiseFactory: promiseFactory
            }, instanceId = QueryStringUtil.getQueryStringValue("instanceId"), telemetryService = visualSandbox.createSandboxTelemetryProxyService(messageProxy), promiseHandler = new SandboxPromiseHandler(), hostServicesObj = visualSandbox.createVisualHostServices(messageProxy, instanceId, promiseHandler), services = {
                $element: element,
                hostServices: hostServicesObj,
                hostServicesControl: hostServicesObj,
                dataProxy: createDataProxy(promiseFactory),
                promiseFactory: promiseFactory,
                messageProxy: messageProxy,
                resourceLoader: powerbi.visualHost.createResourceLoader(resourceLoaderConstructionOptions, null),
                resourceProvider: resourceProvider,
                telemetryProxyService: telemetryService,
                instanceId: instanceId,
                sandboxPromiseHandler: promiseHandler
            };
            return new SandboxHost(services, isolate);
        }
        function createDataProxy(promiseFactory) {
            var noOpTelemetryService = visualSandbox.createSandboxNoOpTelemetryProxyService(), plugins = {
                dsr: {
                    name: "dsr",
                    reader: function(h) {
                        return new powerbi.data.dsr.DsrDataReader(h, {
                            execute: function() {
                                return promiseFactory.reject();
                            }
                        }, noOpTelemetryService, {
                            enableCellLevelFormatting: !1,
                            dsrV2: !1
                        }, null, (void 0), (void 0));
                    }
                }
            }, hostServices = {
                promiseFactory: function() {
                    return promiseFactory;
                }
            }, dataReaderProvider = powerbi.data.createDataReaderProvider(powerbi.data.createDataReaderFactory(plugins, {}), hostServices);
            return powerbi.data.createDataProxy(promiseFactory, dataReaderProvider, noOpTelemetryService);
        }
        function injectCustomVisual(plugin, noIframe, isolate) {
            var content = plugin.content;
            noIframe && plugin.hasBeenInjected || (powerbi.visualHost.resourceInjector.injectJsCode(plugin.name, content.js, isolate, plugin.apiVersion), 
            powerbi.visualHost.resourceInjector.injectCssCode(plugin.name, content.css));
            var injectedPlugin = powerbi.visuals.plugins[plugin.name];
            return plugin.apiVersion && (injectedPlugin.iconUrl = plugin.iconUrl, injectedPlugin.capabilities = plugin.capabilities, 
            injectedPlugin.localizationResources = plugin.localizationResources), noIframe && (injectedPlugin.hasBeenInjected = !0, 
            injectedPlugin.approvedForNoIframe = !0), injectedPlugin;
        }
        var DataViewMerger = powerbi.data.segmentation.DataViewMerger, DataViewObjectSerializer = powerbi.data.services.DataViewObjectSerializer, DataViewObjectDescriptorSerializer = powerbi.data.services.DataViewObjectDescriptorSerializer, VisualObjectInstanceSerializer = powerbi.visualHost.services.VisualObjectInstanceSerializer, VisualObjectRepetitionSerializer = powerbi.visualHost.services.VisualObjectRepetitionSerializer, VisualStyleSerializer = powerbi.visualHost.VisualStyleSerializer, createVisualApiAdapter = powerbi.extensibility.createVisualApiAdapter, QueryStringUtil = jsCommon.QueryStringUtil, Version = jsCommon.Version;
        visualSandbox.relayMouseEvents = [ "click", "mouseup", "MSPointerUp", "pointerup" ], 
        visualSandbox.relayTouchEvents = [ "tap", "touchend", "touchcancel" ], visualSandbox.relayUIEvents = visualSandbox.relayMouseEvents.concat(visualSandbox.relayTouchEvents), 
        visualSandbox.relayUIEventProps = [ "type", "bubbles", "which", "buttons", "button", "ctrlKey", "altKey", "metaKey", "shiftKey", "x", "y", "clientX", "clientY", "pageX", "pageY", "movementX", "movementY", "layerX", "layerY", "changedTouches" ];
        var AGGREGATED_DATAVIEW_SIZE_LIMIT = 104857600, SandboxPromise = function() {
            function SandboxPromise(deferred) {
                this.id = ++SandboxPromise.counter, this.deferred = deferred;
            }
            return SandboxPromise.getLastId = function() {
                return SandboxPromise.counter;
            }, SandboxPromise.counter = 0, SandboxPromise;
        }();
        visualSandbox.SandboxPromise = SandboxPromise;
        var SandboxPromise2 = function() {
            function SandboxPromise2(deferred) {
                this.id = ++SandboxPromise2.counter, this.deferred = deferred;
            }
            return SandboxPromise2.getLastId = function() {
                return SandboxPromise2.counter;
            }, SandboxPromise2.counter = 0, SandboxPromise2;
        }();
        visualSandbox.SandboxPromise2 = SandboxPromise2;
        var SandboxPromiseHandler = function() {
            function SandboxPromiseHandler() {
                this.promises = {}, this.promises2 = {};
            }
            return SandboxPromiseHandler.prototype.CreatePromise = function(deferred) {
                var promise = new SandboxPromise(deferred);
                return this.promises[promise.id] = promise, promise;
            }, SandboxPromiseHandler.prototype.ResolvePromise = function(id, value) {
                var promise = this.promises[id];
                promise && (promise.deferred.resolve(value), this.promises[id] = null);
            }, SandboxPromiseHandler.prototype.RejectPromise = function(id, reason) {
                var promise = this.promises[id];
                promise && (promise.deferred.reject(reason), this.promises[id] = null);
            }, SandboxPromiseHandler.prototype.CreatePromise2 = function(deferred) {
                var promise2 = new SandboxPromise2(deferred);
                return this.promises2[promise2.id] = promise2, promise2;
            }, SandboxPromiseHandler.prototype.ResolvePromise2 = function(id, value) {
                var promise2 = this.promises2[id];
                promise2 && (promise2.deferred.resolve(value), this.promises2[id] = null);
            }, SandboxPromiseHandler.prototype.RejectPromise2 = function(id, reason) {
                var promise2 = this.promises2[id];
                promise2 && (promise2.deferred.reject(reason), this.promises2[id] = null);
            }, SandboxPromiseHandler;
        }();
        visualSandbox.SandboxPromiseHandler = SandboxPromiseHandler, visualSandbox.createSandboxHost = createSandboxHost, 
        visualSandbox.injectCustomVisual = injectCustomVisual;
        var SandboxHost = function() {
            function SandboxHost(services, isolate) {
                void 0 === isolate && (isolate = !1);
                var _this = this;
                this.isolate = isolate, this.isVisualInitialized = !1, this.messageQueue = [], this.relayUIEventListener = function(e) {
                    var DOMEvent = _.pick(e.originalEvent, visualSandbox.relayUIEventProps), message = {
                        eventName: "onRelayUIEvent",
                        arguments: [ DOMEvent ]
                    };
                    _this.messageProxy.postMessage(message);
                }, this.container = services.$element, this.hostServices = services.hostServices, 
                this.hostServicesControl = services.hostServicesControl, this.dataProxy = services.dataProxy, 
                this.promiseFactory = services.promiseFactory, this.messageProxy = services.messageProxy, 
                this.resourceLoader = services.resourceLoader, this.resourceProvider = services.resourceProvider, 
                this.telemetryProxyService = services.telemetryProxyService, this.sandboxPromiseHandler = services.sandboxPromiseHandler, 
                this.aggregatedDataViewSize = 0, this.messageProxy.on(powerbi.visualHost.events.WindowMessageProxyMessageEvent, function(e) {
                    return _this.onMessageReceived(e);
                });
            }
            return Object.defineProperty(SandboxHost.prototype, "visual", {
                get: function() {
                    return this._visual;
                },
                enumerable: !0,
                configurable: !0
            }), SandboxHost.prototype.init = function(options) {
                var initOptions = {
                    animation: options.animation,
                    element: this.container,
                    host: this.hostServices,
                    interactivity: options.interactivity,
                    style: VisualStyleSerializer.deserializeVisualStyle(options.style),
                    viewport: options.viewport,
                    localizationResources: this.plugin.localizationResources
                };
                this.attachRelayUIEventListener(), this.attachWindowResizeListener(), this.featureSwitches = options.featureSwitches, 
                this.visual.init(initOptions);
            }, SandboxHost.prototype.destroy = function() {
                this.detachRelayUIEventListener(), this.detachWindowResizeListener(), this.messageProxy.off(powerbi.visualHost.events.WindowMessageProxyMessageEvent), 
                this.messageProxy.destroy(), this.visual && this.visual.destroy && this.visual.destroy();
            }, SandboxHost.prototype.categoriesExists = function(visualUpdateOptions) {
                return !!(visualUpdateOptions && visualUpdateOptions.dataViews && visualUpdateOptions.dataViews.length > 0 && visualUpdateOptions.dataViews[0].categorical && visualUpdateOptions.dataViews[0].categorical.categories && visualUpdateOptions.dataViews[0].categorical.categories.length > 0 && visualUpdateOptions.dataViews[0].categorical.categories[0] && visualUpdateOptions.dataViews[0].categorical.categories[0].values);
            }, SandboxHost.prototype.getHostCategoriesValues = function(categoriesValues, visualUpdateOptions, isCategoryDate) {
                try {
                    if (categoriesValues) {
                        if (isCategoryDate) for (var i = 0; i < categoriesValues.length; i++) categoriesValues[i] && (categoriesValues[i] = new Date(categoriesValues[i]));
                        visualUpdateOptions.dataViews[0].categorical.categories[0].values = categoriesValues;
                    }
                } catch (e) {
                    this.sendError(e);
                }
                return visualUpdateOptions;
            }, SandboxHost.prototype.update = function(eventArgs) {
                var dataTransforms, _this = this, options = eventArgs.message.arguments[0];
                if (this.plugin && this.plugin.apiVersion && !Version.lt(this.plugin.apiVersion, "2.0.0")) {
                    var DataViews = options.serializedDataView ? [ visualSandbox.DataViewUtils.deserializeDataView(options.serializedDataView) ] : [], visualUpdateOptions = this.lastVisualUpdateOptions = {
                        dataViews: DataViews,
                        suppressAnimations: options.suppressAnimations,
                        viewMode: options.viewMode,
                        editMode: options.editMode,
                        viewport: options.viewport,
                        type: options.type,
                        operationKind: powerbi.VisualDataChangeOperationKind.Create,
                        resizeMode: options.resizeMode,
                        isInFocus: options.isInFocus,
                        jsonFilters: options.jsonFilters ? options.jsonFilters : null
                    };
                    this.fixUpdateOptionsAndUpdateVisual(visualUpdateOptions, eventArgs, options);
                } else options.dataTransforms && (dataTransforms = powerbi.data.services.DataViewTransformActionsSerializer.deserializeDataViewTransformActions(options.dataTransforms, this.plugin.capabilities.objects || {})), 
                this.transformDataViewSource(options.dataViewSource, dataTransforms, !0).then(function(result) {
                    var visualUpdateOptions = _this.lastVisualUpdateOptions = {
                        dataViews: result.dataViews,
                        suppressAnimations: options.suppressAnimations,
                        viewMode: options.viewMode,
                        editMode: options.editMode,
                        viewport: options.viewport,
                        type: options.type,
                        operationKind: powerbi.VisualDataChangeOperationKind.Create,
                        resizeMode: options.resizeMode,
                        isInFocus: options.isInFocus,
                        jsonFilters: options.jsonFilters ? options.jsonFilters : null
                    };
                    _this.fixUpdateOptionsAndUpdateVisual(visualUpdateOptions, eventArgs, options);
                }).catch(function(err) {
                    return _this.sendError(err);
                });
            }, SandboxHost.prototype.updateWithCachedData = function(eventArgs) {
                var dataTransforms, dataViews, options = eventArgs.message.arguments[0];
                this.plugin && this.plugin.apiVersion && !Version.lt(this.plugin.apiVersion, "2.0.0") ? (this.lastVisualUpdateOptions.dataViews && this.lastVisualUpdateOptions.dataViews[0] && this.lastVisualUpdateOptions.dataViews[0].metadata && (this.lastVisualUpdateOptions.dataViews[0].metadata = options.dataViewMetadata ? JSON.parse(options.dataViewMetadata) : void 0), 
                dataViews = this.lastVisualUpdateOptions.dataViews) : (options.dataTransforms && (dataTransforms = powerbi.data.services.DataViewTransformActionsSerializer.deserializeDataViewTransformActions(options.dataTransforms, this.plugin.capabilities.objects || {})), 
                this.lastDataViewTransformApplyOptions.transforms = dataTransforms, dataViews = powerbi.data.DataViewTransform.apply(this.lastDataViewTransformApplyOptions));
                var visualUpdateOptions = this.lastVisualUpdateOptions = {
                    dataViews: dataViews,
                    suppressAnimations: options.suppressAnimations,
                    viewMode: options.viewMode,
                    editMode: options.editMode,
                    viewport: options.viewport,
                    type: options.type,
                    operationKind: powerbi.VisualDataChangeOperationKind.Create,
                    resizeMode: options.resizeMode,
                    isInFocus: options.isInFocus,
                    jsonFilters: options.jsonFilters ? options.jsonFilters : null
                };
                this.fixUpdateOptionsAndUpdateVisual(visualUpdateOptions, eventArgs, options);
            }, SandboxHost.prototype.loadResourcePackage = function(resourcePackage) {
                var _this = this;
                this.resourceLoader.loadVisualPlugin(resourcePackage).then(function(plugin) {
                    _this.plugin = plugin, _this.createAdapter();
                }).catch(function() {
                    var error = new Error("Unable to load plugin, resource package id: " + resourcePackage.id);
                    return _this.sendError(error);
                });
            }, SandboxHost.prototype.createAdapter = function() {
                var plugin = this.plugin;
                if (this.container.addClass("visual-" + plugin.name), this._visual = createVisualApiAdapter(plugin, plugin.create, this.promiseFactory, this.telemetryProxyService), 
                !plugin || !this.visual) {
                    var error = new Error("Unable to create visual from plugin " + plugin.name);
                    return this.sendError(error);
                }
                this.flushQueue(), this.isVisualInitialized = !0, this.sendSandboxReady();
            }, SandboxHost.prototype.loadWithoutResourcePackage = function(plugin) {
                this.plugin = injectCustomVisual(plugin, !1, this.isolate), powerbi.extensibility.visualPluginAdapter.transformPlugin(this.plugin), 
                this.createAdapter();
            }, SandboxHost.prototype.enumerateObjectInstances = function(options) {
                if (this.visual.enumerateObjectInstances) {
                    var visualObjectInstances = this.visual.enumerateObjectInstances(options);
                    if (visualObjectInstances) {
                        var visualObjectEnumeration = visualObjectInstances, dontSerialize = this.plugin && this.plugin.apiVersion && !Version.lt(this.plugin.apiVersion, "2.0.0");
                        return visualObjectEnumeration && visualObjectEnumeration.instances ? dontSerialize ? {
                            instances: visualObjectEnumeration.instances,
                            containers: visualObjectEnumeration.containers
                        } : {
                            instances: VisualObjectInstanceSerializer.serialize(visualObjectEnumeration.instances),
                            containers: visualObjectEnumeration.containers ? VisualObjectInstanceSerializer.serializeContainers(visualObjectEnumeration.containers, this.resourceProvider) : null
                        } : dontSerialize ? visualObjectInstances : VisualObjectInstanceSerializer.serialize(visualObjectInstances);
                    }
                }
            }, SandboxHost.prototype.enumerateObjectRepetition = function() {
                if (this.visual.enumerateObjectRepetition) {
                    var visualObjectRepetition = this.visual.enumerateObjectRepetition();
                    return visualObjectRepetition ? VisualObjectRepetitionSerializer.serialize(visualObjectRepetition) : void 0;
                }
            }, SandboxHost.prototype.sendUpdateResult = function() {
                var containerChildCount = this.container.children("*").length, message = {
                    eventName: "onUpdateResult",
                    arguments: [ containerChildCount ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxHost.prototype.sendSandboxReady = function() {
                var capabilities = this.plugin.capabilities;
                capabilities.objects && (capabilities.objects = DataViewObjectDescriptorSerializer.serialize(this.plugin.capabilities.objects, this.resourceProvider));
                var proxyPlugin = {
                    name: this.plugin.name,
                    watermarkKey: this.plugin.watermarkKey,
                    capabilities: capabilities,
                    class: this.plugin.class,
                    iconUrl: this.plugin.iconUrl,
                    custom: !0,
                    create: function() {
                        return null;
                    }
                }, message = {
                    eventName: "onPluginReady",
                    arguments: [ proxyPlugin ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxHost.prototype.flushQueue = function() {
                for (;this.messageQueue.length > 0; ) {
                    var message = this.messageQueue.shift();
                    this.executeMessage(message);
                }
            }, SandboxHost.prototype.sendError = function(error, originMessage) {
                var message = {
                    eventName: "onError",
                    arguments: [],
                    error: {
                        id: originMessage ? originMessage.id : null,
                        eventName: originMessage ? originMessage.eventName : null,
                        name: error.name,
                        message: error.message,
                        pluginName: this.plugin.name
                    }
                };
                this.messageProxy.postMessage(message);
            }, SandboxHost.prototype.onMessageReceived = function(e) {
                return this.isVisualInitialized || "loadResourcePackage" === e.message.eventName || "loadWithoutResourcePackage" === e.message.eventName ? this.executeMessage(e) : void this.messageQueue.push(e);
            }, SandboxHost.prototype.executeMessage = function(e) {
                var message = e.message;
                try {
                    if ($.isFunction(this[message.eventName])) return "update" === message.eventName || "updateWithCachedData" === message.eventName || "handleLoadMoreDataResult" === message.eventName ? this[message.eventName].call(this, e) : this[message.eventName].apply(this, message.arguments);
                    if (this.visual && $.isFunction(this.visual[message.eventName])) return this.visual[message.eventName].apply(this.visual, message.arguments);
                } catch (ex) {
                    this.sendError(ex, message);
                }
            }, SandboxHost.prototype.transformDataViewSource = function(dataViewSource, dataTransforms, bypassConcatenation) {
                var _this = this, deferred = this.promiseFactory.defer(), createDataViewPromise = dataViewSource ? this.createDataView(dataViewSource) : this.promiseFactory.resolve({
                    dataView: null
                });
                return createDataViewPromise.then(function(result) {
                    var dataViews = powerbi.data.DataViewTransform.apply(_this.lastDataViewTransformApplyOptions = {
                        prototype: result.dataView,
                        objectDescriptors: _this.plugin.capabilities ? _this.plugin.capabilities.objects : void 0,
                        dataViewMappings: _this.plugin.capabilities ? _this.plugin.capabilities.dataViewMappings : void 0,
                        dataRoles: _this.plugin.capabilities ? _this.plugin.capabilities.dataRoles : void 0,
                        drillCapabilities: _this.plugin.capabilities && _this.plugin.capabilities.drilldown,
                        expandCollapseCapabilties: _this.plugin.capabilities && _this.plugin.capabilities.expandCollapse,
                        transforms: dataTransforms,
                        colorAllocatorFactory: powerbi.visuals.createColorAllocatorFactory(),
                        bypassConcatenation: bypassConcatenation,
                        visualStyle: powerbi.visuals.visualStyles.create()
                    });
                    deferred.resolve({
                        dataViews: dataViews
                    });
                }).catch(function(err) {
                    return _this.sendError(err);
                }), deferred.promise;
            }, SandboxHost.prototype.createDataView = function(dataViewSource) {
                var _this = this, deferred = this.promiseFactory.defer(), options = {
                    type: dataViewSource.type,
                    query: {
                        dataSources: null,
                        command: dataViewSource.data
                    }
                };
                return this.dataProxy.execute(options).then(function(result) {
                    var dataView = result && result.dataReaderResult ? result.dataReaderResult.dataView : null, validation = powerbi.DataViewAnalysis.validateAndReshape(dataView, _this.plugin.capabilities ? _this.plugin.capabilities.dataViewMappings : null);
                    deferred.resolve({
                        dataView: validation.dataView
                    });
                }).catch(function(err) {
                    return _this.sendError(err);
                }), deferred.promise;
            }, SandboxHost.prototype.attachWindowResizeListener = function() {
                $(window).on("resize", this.windowResizeListener.bind(this)), $(window).trigger("resize");
            }, SandboxHost.prototype.detachWindowResizeListener = function() {
                $(window).off("resize");
            }, SandboxHost.prototype.windowResizeListener = function() {
                if (this.lastVisualUpdateOptions) {
                    var height = $(window).height(), width = $(window).width(), visualUpdateOptions = powerbi.Prototype.inherit(this.lastVisualUpdateOptions);
                    visualUpdateOptions.viewport = {
                        width: width,
                        height: height
                    }, visualUpdateOptions.type = powerbi.VisualUpdateType.Resize, visualUpdateOptions.resizeMode = 1, 
                    this.visual.update(visualUpdateOptions);
                }
            }, SandboxHost.prototype.attachRelayUIEventListener = function() {
                this.container.on(visualSandbox.relayUIEvents.join(" "), this.relayUIEventListener);
            }, SandboxHost.prototype.detachRelayUIEventListener = function() {
                this.container.off(visualSandbox.relayUIEvents.join(" "), this.relayUIEventListener);
            }, SandboxHost.prototype.handleLoadMoreDataResult = function(eventArgs) {
                if (this.plugin && this.plugin.apiVersion && !Version.lt(this.plugin.apiVersion, "2.0.0")) {
                    var serializedDataView = eventArgs.message.arguments[0];
                    if (serializedDataView) {
                        var dataViews = [ visualSandbox.DataViewUtils.deserializeDataView(serializedDataView) ];
                        DataViewMerger.mergeDataViews(this.lastVisualUpdateOptions.dataViews[0], dataViews[0]), 
                        this.lastVisualUpdateOptions.operationKind = powerbi.VisualDataChangeOperationKind.Append;
                    }
                } else {
                    var result = eventArgs.message.arguments[0], segment = result ? result.dataReaderResult.dataView : null;
                    segment && (DataViewMerger.mergeDataViews(this.lastDataViewTransformApplyOptions.prototype, segment), 
                    this.lastVisualUpdateOptions.dataViews = powerbi.data.DataViewTransform.apply(this.lastDataViewTransformApplyOptions), 
                    this.lastVisualUpdateOptions.operationKind = powerbi.VisualDataChangeOperationKind.Append);
                }
                this.aggregatedDataViewSize += eventArgs.packedSize, this.aggregatedDataViewSize >= AGGREGATED_DATAVIEW_SIZE_LIMIT ? this.hostServicesControl.enableLoadMoreData(!1) : this.hostServicesControl.enableLoadMoreData(!0), 
                this.visual.update(this.lastVisualUpdateOptions);
                var message = {
                    eventName: "stopProgressBar",
                    arguments: []
                };
                this.messageProxy.postMessage(message);
            }, SandboxHost.prototype.handleGetAADTokenResult = function(result) {
                result.isError ? this.sandboxPromiseHandler.RejectPromise(result.promiseId, result.payload) : this.sandboxPromiseHandler.ResolvePromise(result.promiseId, result.payload);
            }, SandboxHost.prototype.handleGetWACTokenResult = function(result) {
                result.isError ? this.sandboxPromiseHandler.RejectPromise2(result.promiseId, result.payload ? result.payload : "") : this.sandboxPromiseHandler.ResolvePromise2(result.promiseId, result.wacToken);
            }, SandboxHost.prototype.handleGetStorageData = function(response) {
                response.data ? this.sandboxPromiseHandler.ResolvePromise(response.promiseId, response.data) : this.sandboxPromiseHandler.RejectPromise(response.promiseId);
            }, SandboxHost.prototype.handleSetStorageData = function(response) {
                response.isError ? this.sandboxPromiseHandler.ResolvePromise(response.promiseId, response.freeSpace) : this.sandboxPromiseHandler.RejectPromise(response.promiseId);
            }, SandboxHost.prototype.onRestoreSelection = function(options) {
                var selectionManager = this.hostServices.selectionManager;
                if (selectionManager) if (options && options.selection) {
                    var selectionIds = options.selection.map(function(selector) {
                        return powerbi.visuals.SelectionId.fromSelectorsByColumn(DataViewObjectSerializer.deserializeSelectorsByColumn(selector));
                    });
                    selectionManager.setSelectionIds(selectionIds), selectionManager.invokeOnSelectCallback(selectionIds);
                } else selectionManager.setSelectionIds([]), selectionManager.invokeOnSelectCallback([]);
            }, SandboxHost.prototype.onRestoreCvSelectionIds = function(options) {
                var selectionManager = this.hostServices.selectionManager;
                if (selectionManager) if (options) {
                    var selectionIds = _.map(options, function(restoreCvOption) {
                        return new powerbi.visuals.CustomVisualSelectionId(restoreCvOption.serializedDataMap, restoreCvOption.measures);
                    });
                    selectionManager.setSelectionIds(selectionIds), selectionManager.invokeOnSelectCallback(selectionIds);
                } else selectionManager.setSelectionIds([]), selectionManager.invokeOnSelectCallback([]);
            }, SandboxHost.prototype.fixUpdateOptionsAndUpdateVisual = function(visualUpdateOptions, eventArgs, options) {
                this.categoriesExists(visualUpdateOptions) && (visualUpdateOptions = this.getHostCategoriesValues(options.categoriesValues, visualUpdateOptions, options.isCategoryDate)), 
                this.hostServicesControl.enableLoadMoreData(!0), this.aggregatedDataViewSize = eventArgs.packedSize, 
                this.visual.update(visualUpdateOptions), this.sendUpdateResult();
            }, SandboxHost;
        }();
        visualSandbox.SandboxHost = SandboxHost;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxVisualHostTooltipService(messageProxy) {
            return new SandboxVisualHostTooltipService(messageProxy);
        }
        var DataServices = powerbi.data.services, DataViewObjectSerializer = DataServices.DataViewObjectSerializer, CustomVisualSelectionId = powerbi.visuals.CustomVisualSelectionId;
        visualSandbox.createSandboxVisualHostTooltipService = createSandboxVisualHostTooltipService;
        var SandboxVisualHostTooltipService = function() {
            function SandboxVisualHostTooltipService(messageProxy) {
                this.messageProxy = messageProxy, this.rootElement = window.document.body;
            }
            return SandboxVisualHostTooltipService.prototype.serializeIdentities = function(identities) {
                if (identities && identities[0] instanceof CustomVisualSelectionId) return identities;
                var serializedIdentities = identities && _(identities).map(function(value) {
                    return DataViewObjectSerializer.serializeSelectorsByColumn(value);
                }).compact().value();
                return serializedIdentities;
            }, SandboxVisualHostTooltipService.prototype.show = function(args) {
                var proxyArgs = args && {
                    coordinates: args.coordinates,
                    dataItems: args.dataItems,
                    identities: this.serializeIdentities(args.identities),
                    isTouchEvent: args.isTouchEvent
                }, message = {
                    eventName: "showTooltip",
                    arguments: [ proxyArgs ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostTooltipService.prototype.move = function(args) {
                var proxyArgs = args && {
                    coordinates: args.coordinates,
                    dataItems: args.dataItems,
                    identities: this.serializeIdentities(args.identities),
                    isTouchEvent: args.isTouchEvent
                }, message = {
                    eventName: "moveTooltip",
                    arguments: [ proxyArgs ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostTooltipService.prototype.hide = function(args) {
                var message = {
                    eventName: "hideTooltip",
                    arguments: [ args ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostTooltipService.prototype.visible = function() {
                return !1;
            }, SandboxVisualHostTooltipService.prototype.container = function() {
                return this.rootElement;
            }, SandboxVisualHostTooltipService.prototype.enabled = function() {
                return !0;
            }, SandboxVisualHostTooltipService;
        }();
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxVisualHostTelemetryService(messageProxy, instanceId) {
            return new SandboxVisualHostTelemetryService(messageProxy, instanceId);
        }
        var convertVisualEventType = powerbi.extensibility.convertVisualEventType;
        visualSandbox.createSandboxVisualHostTelemetryService = createSandboxVisualHostTelemetryService;
        var SandboxVisualHostTelemetryService = function() {
            function SandboxVisualHostTelemetryService(messageProxy, instanceId) {
                this.messageProxy = messageProxy, this.instanceId = instanceId;
            }
            return SandboxVisualHostTelemetryService.prototype.trace = function(type, payload) {
                this.messageProxy.postMessageAsync({
                    eventName: "logVisualEvent",
                    arguments: [ convertVisualEventType(type), this.instanceId, payload ]
                });
            }, SandboxVisualHostTelemetryService;
        }();
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxVisualHostLocalStorageService(messageProxy, promiseFactory, sandboxPromiseHandler) {
            return new SandboxVisualHostLocalStorageService(messageProxy, promiseFactory, sandboxPromiseHandler);
        }
        visualSandbox.createSandboxVisualHostLocalStorageService = createSandboxVisualHostLocalStorageService;
        var SandboxVisualHostLocalStorageService = function() {
            function SandboxVisualHostLocalStorageService(messageProxy, promiseFactory, sandboxPromiseHandler) {
                this.messageProxy = messageProxy, this.promiseFactory = promiseFactory, this.sandboxPromiseHandler = sandboxPromiseHandler;
            }
            return SandboxVisualHostLocalStorageService.prototype.getStorageData = function(key) {
                var promise = this.sandboxPromiseHandler.CreatePromise(this.promiseFactory.defer()), message = {
                    eventName: "getStorageData",
                    arguments: [ key, promise.id ]
                };
                return this.messageProxy.postMessage(message), promise.deferred.promise;
            }, SandboxVisualHostLocalStorageService.prototype.setStorageData = function(key, data) {
                var promise = this.sandboxPromiseHandler.CreatePromise(this.promiseFactory.defer()), message = {
                    eventName: "setStorageData",
                    arguments: [ key, data, promise.id ]
                };
                return this.messageProxy.postMessage(message), promise.deferred.promise;
            }, SandboxVisualHostLocalStorageService.prototype.removeStorageData = function(key) {
                var message = {
                    eventName: "removeStorageData",
                    arguments: [ key ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostLocalStorageService;
        }();
        visualSandbox.SandboxVisualHostLocalStorageService = SandboxVisualHostLocalStorageService;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxVisualHostEventService(messageProxy, instanceId) {
            return new SandboxVisualHostEventService(messageProxy, instanceId);
        }
        visualSandbox.createSandboxVisualHostEventService = createSandboxVisualHostEventService;
        var SandboxVisualHostEventService = function() {
            function SandboxVisualHostEventService(messageProxy, instanceId) {
                this.messageProxy = messageProxy, this.instanceId = instanceId;
            }
            return SandboxVisualHostEventService.prototype.renderingStarted = function(options) {
                var message = {
                    eventName: "renderingStarted",
                    arguments: [ this.instanceId, options.updateId ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostEventService.prototype.renderingFinished = function(options) {
                var message = {
                    eventName: "renderingFinished",
                    arguments: [ this.instanceId, options.updateId ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostEventService.prototype.renderingFailed = function(options, reason) {
                var message = {
                    eventName: "renderingFailed",
                    arguments: [ this.instanceId, options.updateId, reason ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostEventService;
        }();
        visualSandbox.SandboxVisualHostEventService = SandboxVisualHostEventService;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxTelemetryProxyService(windowMessageProxy) {
            return new SandboxTelemetryProxyService(windowMessageProxy);
        }
        var proxiedTelemetryEventType;
        !function(proxiedTelemetryEventType) {
            proxiedTelemetryEventType.VisualApiUsage = "VisualApiUsage", proxiedTelemetryEventType.VisualException = "VisualException";
        }(proxiedTelemetryEventType = visualSandbox.proxiedTelemetryEventType || (visualSandbox.proxiedTelemetryEventType = {})), 
        visualSandbox.createSandboxTelemetryProxyService = createSandboxTelemetryProxyService;
        var SandboxTelemetryProxyService = function() {
            function SandboxTelemetryProxyService(windowMessageProxy) {
                this.windowMessageProxy = windowMessageProxy;
            }
            return SandboxTelemetryProxyService.prototype.setLogToConsole = function(enabled) {}, 
            SandboxTelemetryProxyService.prototype.setLogAllEvents = function(enabled) {}, SandboxTelemetryProxyService.prototype.suspend = function() {}, 
            SandboxTelemetryProxyService.prototype.resume = function() {}, SandboxTelemetryProxyService.prototype.flush = function() {}, 
            SandboxTelemetryProxyService.prototype.restart = function() {}, SandboxTelemetryProxyService.prototype.startPerfEventDeprecated = function(scope, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new EmptyDeferredTelemetryEvent(event);
            }, SandboxTelemetryProxyService.prototype.logEvent = function(eventFactory) {
                for (var args = [], _i = 1; _i < arguments.length; _i++) args[_i - 1] = arguments[_i];
                var eventType = null;
                return eventFactory === powerbi.telemetry.VisualApiUsage ? eventType = proxiedTelemetryEventType.VisualApiUsage : eventFactory === powerbi.telemetry.VisualException && (eventType = proxiedTelemetryEventType.VisualException), 
                args.unshift(eventType), this.windowMessageProxy.postMessageAsync({
                    eventName: "logEvent",
                    arguments: args
                }), null;
            }, SandboxTelemetryProxyService.prototype.logChildEvent = function(parent, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
            }, SandboxTelemetryProxyService.prototype.startActivity = function(scope, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new EmptyDeferredTelemetryEvent(event);
            }, SandboxTelemetryProxyService.prototype.startEvent = function(eventFactory) {
                for (var args = [], _i = 1; _i < arguments.length; _i++) args[_i - 1] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new EmptyDeferredTelemetryEvent(event);
            }, SandboxTelemetryProxyService.prototype.startChildEvent = function(parent, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new EmptyDeferredTelemetryEvent(event);
            }, SandboxTelemetryProxyService.prototype.logTrace = function(trace) {}, Object.defineProperty(SandboxTelemetryProxyService.prototype, "sessionId", {
                get: function() {},
                enumerable: !0,
                configurable: !0
            }), Object.defineProperty(SandboxTelemetryProxyService.prototype, "root", {
                get: function() {},
                enumerable: !0,
                configurable: !0
            }), SandboxTelemetryProxyService.prototype.setHostInfo = function(key, value) {}, 
            SandboxTelemetryProxyService;
        }(), EmptyDeferredTelemetryEvent = function() {
            function EmptyDeferredTelemetryEvent(event) {
                var _this = this;
                this.event = event, this.promise = new powerbi.Promise(function(r, rej) {
                    _this.onResolve = r, _this.onReject = rej;
                });
            }
            return EmptyDeferredTelemetryEvent.prototype.resolve = function() {
                this.onResolve();
            }, EmptyDeferredTelemetryEvent.prototype.reject = function(errorDetails) {
                this.onReject();
            }, EmptyDeferredTelemetryEvent;
        }();
        visualSandbox.EmptyDeferredTelemetryEvent = EmptyDeferredTelemetryEvent;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxNoOpTelemetryProxyService() {
            return new SandboxNoOpTelemetryProxyService();
        }
        visualSandbox.createSandboxNoOpTelemetryProxyService = createSandboxNoOpTelemetryProxyService;
        var SandboxNoOpTelemetryProxyService = function() {
            function SandboxNoOpTelemetryProxyService() {}
            return SandboxNoOpTelemetryProxyService.prototype.setLogToConsole = function(enabled) {}, 
            SandboxNoOpTelemetryProxyService.prototype.setLogAllEvents = function(enabled) {}, 
            SandboxNoOpTelemetryProxyService.prototype.suspend = function() {}, SandboxNoOpTelemetryProxyService.prototype.resume = function() {}, 
            SandboxNoOpTelemetryProxyService.prototype.flush = function() {}, SandboxNoOpTelemetryProxyService.prototype.restart = function() {}, 
            SandboxNoOpTelemetryProxyService.prototype.startPerfEventDeprecated = function(scope, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new visualSandbox.EmptyDeferredTelemetryEvent(event);
            }, SandboxNoOpTelemetryProxyService.prototype.logEvent = function(eventFactory) {
                for (var args = [], _i = 1; _i < arguments.length; _i++) args[_i - 1] = arguments[_i];
                return null;
            }, SandboxNoOpTelemetryProxyService.prototype.logChildEvent = function(parent, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
            }, SandboxNoOpTelemetryProxyService.prototype.startActivity = function(scope, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new visualSandbox.EmptyDeferredTelemetryEvent(event);
            }, SandboxNoOpTelemetryProxyService.prototype.startEvent = function(eventFactory) {
                for (var args = [], _i = 1; _i < arguments.length; _i++) args[_i - 1] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new visualSandbox.EmptyDeferredTelemetryEvent(event);
            }, SandboxNoOpTelemetryProxyService.prototype.startChildEvent = function(parent, eventFactory) {
                for (var args = [], _i = 2; _i < arguments.length; _i++) args[_i - 2] = arguments[_i];
                var tempEvent, event = {
                    info: {}
                };
                return tempEvent = new visualSandbox.EmptyDeferredTelemetryEvent(event);
            }, SandboxNoOpTelemetryProxyService.prototype.logTrace = function(trace) {}, Object.defineProperty(SandboxNoOpTelemetryProxyService.prototype, "sessionId", {
                get: function() {},
                enumerable: !0,
                configurable: !0
            }), Object.defineProperty(SandboxNoOpTelemetryProxyService.prototype, "root", {
                get: function() {},
                enumerable: !0,
                configurable: !0
            }), SandboxNoOpTelemetryProxyService.prototype.setHostInfo = function(key, value) {}, 
            SandboxNoOpTelemetryProxyService;
        }();
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createVisualHostServices(messageProxy, instanceId, sandboxPromiseHandler) {
            return new SandboxVisualHostServices(messageProxy, instanceId, sandboxPromiseHandler);
        }
        var DataViewObjectSerializer = powerbi.data.services.DataViewObjectSerializer, VisualObjectInstanceSerializer = powerbi.visualHost.services.VisualObjectInstanceSerializer, Version = jsCommon.Version, BeautifiedFormat = {
            "0.00 %;-0.00 %;0.00 %": "Percentage",
            "0.0 %;-0.0 %;0.0 %": "Percentage1"
        }, defaultLocalizedStrings = {
            NullValue: "(Blank)",
            BooleanTrue: "True",
            BooleanFalse: "False",
            NaNValue: "NaN",
            InfinityValue: "+Infinity",
            NegativeInfinityValue: "-Infinity",
            Restatement_Comma: "{0}, {1}",
            Restatement_CompoundAnd: "{0} and {1}",
            DisplayUnitSystem_EAuto_Title: "Auto",
            DisplayUnitSystem_E0_Title: "None",
            DisplayUnitSystem_E3_LabelFormat: "{0}K",
            DisplayUnitSystem_E3_Title: "Thousands",
            DisplayUnitSystem_E6_LabelFormat: "{0}M",
            DisplayUnitSystem_E6_Title: "Millions",
            DisplayUnitSystem_E9_LabelFormat: "{0}bn",
            DisplayUnitSystem_E9_Title: "Billions",
            DisplayUnitSystem_E12_LabelFormat: "{0}T",
            DisplayUnitSystem_E12_Title: "Trillions",
            Percentage: "#,0.##%",
            Percentage1: "#,0.#%",
            TableTotalLabel: "Total",
            Tooltip_HighlightedValueDisplayName: "Highlighted",
            Waterfall_DecreaseLabel: "Decrease",
            Waterfall_IncreaseLabel: "Increase",
            Waterfall_TotalLabel: "Total",
            GeotaggingString_Continent: "continent",
            GeotaggingString_Continents: "continents",
            GeotaggingString_Country: "country",
            GeotaggingString_Countries: "countries",
            GeotaggingString_State: "state",
            GeotaggingString_States: "states",
            GeotaggingString_City: "city",
            GeotaggingString_Cities: "cities",
            GeotaggingString_Town: "town",
            GeotaggingString_Towns: "towns",
            GeotaggingString_Province: "province",
            GeotaggingString_Provinces: "provinces",
            GeotaggingString_County: "county",
            GeotaggingString_Counties: "counties",
            GeotaggingString_Village: "village",
            GeotaggingString_Villages: "villages",
            GeotaggingString_Post: "post",
            GeotaggingString_Zip: "zip",
            GeotaggingString_Code: "code",
            GeotaggingString_Place: "place",
            GeotaggingString_Places: "places",
            GeotaggingString_Address: "address",
            GeotaggingString_Addresses: "addresses",
            GeotaggingString_Street: "street",
            GeotaggingString_Streets: "streets",
            GeotaggingString_Longitude: "longitude",
            GeotaggingString_Longitude_Short: "lon",
            GeotaggingString_Longitude_Short2: "long",
            GeotaggingString_Latitude: "latitude",
            GeotaggingString_Latitude_Short: "lat",
            GeotaggingString_PostalCode: "postal code",
            GeotaggingString_PostalCodes: "postal codes",
            GeotaggingString_ZipCode: "zip code",
            GeotaggingString_ZipCodes: "zip codes",
            GeotaggingString_Territory: "territory",
            GeotaggingString_Territories: "territories"
        }, SandboxVisualHostServices = function() {
            function SandboxVisualHostServices(messageProxy, instanceId, sandboxPromiseHandler) {
                this.messageProxy = messageProxy, this.loadMoreDataEnabled = !0, this.instanceId = instanceId, 
                this.visualHostTooltipService = visualSandbox.createSandboxVisualHostTooltipService(messageProxy), 
                this.telemetryService = visualSandbox.createSandboxVisualHostTelemetryService(messageProxy, instanceId), 
                this.queryDictionary = SandboxVisualHostServices.parseUrlParameters(), this.authService = visualSandbox.createSandboxVisualHostAuthenticationService(messageProxy, this.promiseFactory(), sandboxPromiseHandler), 
                this.storageService = visualSandbox.createSandboxVisualHostLocalStorageService(messageProxy, this.promiseFactory(), sandboxPromiseHandler), 
                this.eventsService = visualSandbox.createSandboxVisualHostEventService(messageProxy, instanceId);
            }
            return SandboxVisualHostServices.initialize = function() {
                powerbi.visuals.TooltipManager && powerbi.visuals.TooltipManager.setLocalizedStrings(SandboxVisualHostServices.createTooltipLocaleOptions());
            }, SandboxVisualHostServices.createLocaleOptions = function() {
                return {
                    null: defaultLocalizedStrings.NullValue,
                    true: defaultLocalizedStrings.BooleanTrue,
                    false: defaultLocalizedStrings.BooleanFalse,
                    NaN: defaultLocalizedStrings.NaNValue,
                    infinity: defaultLocalizedStrings.InfinityValue,
                    negativeInfinity: defaultLocalizedStrings.NegativeInfinityValue,
                    beautify: function(format) {
                        return SandboxVisualHostServices.beautify(format);
                    },
                    describe: function(exponent) {
                        return SandboxVisualHostServices.describeUnit(exponent);
                    },
                    restatementComma: defaultLocalizedStrings.Restatement_Comma,
                    restatementCompoundAnd: defaultLocalizedStrings.Restatement_CompoundAnd,
                    restatementCompoundOr: defaultLocalizedStrings.Restatement_CompoundOr
                };
            }, SandboxVisualHostServices.createTooltipLocaleOptions = function() {
                return {
                    highlightedValueDisplayName: defaultLocalizedStrings.Tooltip_HighlightedValueDisplayName
                };
            }, SandboxVisualHostServices.prototype.getLocalizedString = function(stringId) {
                return defaultLocalizedStrings[stringId];
            }, SandboxVisualHostServices.prototype.onSelecting = function(selectingArgs) {
                selectingArgs.action = 0;
            }, SandboxVisualHostServices.prototype.onSelect = function(selectArgs) {
                var proxyArgs;
                if (powerbi.extensibility.legacy.isOldSelectEventArgs(selectArgs)) {
                    var deprecatedArgs = selectArgs;
                    proxyArgs = {
                        data: DataViewObjectSerializer.serializeSelectors(deprecatedArgs.data),
                        data2: DataViewObjectSerializer.serializeSelectorsByColumn(deprecatedArgs.data2),
                        selectors: DataViewObjectSerializer.serializeSelectors(selectArgs.selectors)
                    };
                } else proxyArgs = {
                    visualObjects: DataViewObjectSerializer.serializeVisualObjects(selectArgs.visualObjects)
                };
                var message = {
                    eventName: "onSelect",
                    arguments: [ proxyArgs ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.customVisualOnContextMenu = function(args) {
                var message = {
                    eventName: "onCustomVisualContextMenu",
                    arguments: [ args ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.persistProperties = function(changes) {
                var wireChanges, instancesToPersist = changes;
                wireChanges = this.apiVersion && (Version.gt(this.apiVersion, "2.0.0") || Version.eq(this.apiVersion, "2.0.0")) ? changes : instancesToPersist.merge || instancesToPersist.remove || instancesToPersist.replace ? {
                    merge: instancesToPersist.merge ? VisualObjectInstanceSerializer.serialize(instancesToPersist.merge) : null,
                    replace: instancesToPersist.replace ? VisualObjectInstanceSerializer.serialize(instancesToPersist.replace) : null,
                    remove: instancesToPersist.remove ? VisualObjectInstanceSerializer.serialize(instancesToPersist.remove) : null
                } : VisualObjectInstanceSerializer.serialize(changes);
                var message = {
                    eventName: "persistProperties",
                    arguments: [ wireChanges ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.sendSelectionIdsToHost = function(selectArgs) {
                var message = {
                    eventName: "transformCvSelectionIdsAndSelect",
                    arguments: [ selectArgs ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.loadMoreData = function() {
                this.fetchMoreData();
            }, SandboxVisualHostServices.prototype.fetchMoreData = function() {
                if (!this.loadMoreDataEnabled) return !1;
                var message = {
                    eventName: "loadMoreData",
                    arguments: null
                };
                return this.messageProxy.postMessage(message), !0;
            }, SandboxVisualHostServices.prototype.launchUrl = function(url) {
                var message = {
                    eventName: "launchUrl",
                    arguments: [ url ]
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.tooltips = function() {
                return this.visualHostTooltipService;
            }, SandboxVisualHostServices.prototype.telemetry = function() {
                return this.telemetryService;
            }, SandboxVisualHostServices.prototype.authenticationService = function() {
                return this.authService;
            }, SandboxVisualHostServices.prototype.localStorageService = function() {
                return this.storageService;
            }, SandboxVisualHostServices.prototype.eventService = function() {
                return this.eventsService;
            }, SandboxVisualHostServices.prototype.locale = function() {
                var loc = this.queryDictionary && this.queryDictionary.locale;
                return loc || navigator.language;
            }, SandboxVisualHostServices.prototype.loader = function() {
                var _this = this;
                return {
                    require: function() {
                        return _this.promiseFactory().reject();
                    }
                };
            }, SandboxVisualHostServices.prototype.applyJsonFilter = function(filters, objectName, propertyName, action) {
                var args = [ filters, objectName, propertyName ];
                action && args.push(action);
                var message = {
                    eventName: "applyJsonFilter",
                    arguments: args
                };
                return this.messageProxy.postMessage(message), !0;
            }, SandboxVisualHostServices.prototype.getJSONFilter = function(_filter) {
                return this.promiseFactory().reject();
            }, SandboxVisualHostServices.parseUrlParameters = function(queryString) {
                void 0 === queryString && (queryString = window.location.search);
                var queryStringDictionary = {}, search = queryString;
                if (search && "?" === search.substr(0, 1)) for (var pairs = search.substr(1).split("&"), _i = 0, pairs_1 = pairs; _i < pairs_1.length; _i++) {
                    var pair = pairs_1[_i], keyValuePair = pair && pair.split("=");
                    keyValuePair && 2 === keyValuePair.length && (queryStringDictionary[decodeURIComponent(keyValuePair[0])] = decodeURIComponent(keyValuePair[1]));
                }
                return queryStringDictionary;
            }, SandboxVisualHostServices.prototype.destroy = function() {
                this.messageProxy.destroy(), this.messageProxy = null;
            }, SandboxVisualHostServices.prototype.refreshHostData = function() {
                var message = {
                    eventName: "refreshHostData",
                    arguments: null
                };
                this.messageProxy.postMessage(message);
            }, SandboxVisualHostServices.prototype.createSelectionIdBuilder = function() {
                return new powerbi.visuals.CustomVisualSelectionId();
            }, SandboxVisualHostServices.prototype.createTooltipService = function() {
                return powerbi.extensibility.createTooltipService({
                    hostServices: this
                });
            }, SandboxVisualHostServices.prototype.requestFocus = function() {}, SandboxVisualHostServices.prototype.onDragStart = function() {}, 
            SandboxVisualHostServices.prototype.onContextMenu = function() {}, SandboxVisualHostServices.prototype.getVisualLinkProperties = function() {}, 
            SandboxVisualHostServices.prototype.canSelect = function() {
                return !1;
            }, SandboxVisualHostServices.prototype.canExpandCollapse = function(roles) {
                return !1;
            }, SandboxVisualHostServices.prototype.setNodeExpansionState = function(_args) {}, 
            SandboxVisualHostServices.prototype.onCustomSort = function(args) {}, SandboxVisualHostServices.prototype.getViewMode = function() {
                return 0;
            }, SandboxVisualHostServices.prototype.getEditMode = function() {
                return 0;
            }, SandboxVisualHostServices.prototype.setWarnings = function(warnings) {}, SandboxVisualHostServices.prototype.setToolbar = function($toolbar) {}, 
            SandboxVisualHostServices.prototype.shouldRetainSelection = function() {
                return !1;
            }, SandboxVisualHostServices.prototype.geocoder = function() {
                return powerbi.visuals.services.createGeocoder();
            }, SandboxVisualHostServices.prototype.geolocation = function() {
                return powerbi.visuals.services.createGeolocation();
            }, SandboxVisualHostServices.prototype.promiseFactory = function() {
                return powerbi.createJQueryPromiseFactory();
            }, SandboxVisualHostServices.prototype.analyzeFilter = function(options) {}, SandboxVisualHostServices.prototype.getIdentityDisplayNames = function(dentities) {}, 
            SandboxVisualHostServices.prototype.setIdentityDisplayNames = function(displayNamesIdentityPairs) {}, 
            SandboxVisualHostServices.prototype.getUIComponentFactory = function() {}, SandboxVisualHostServices.beautify = function(format) {
                var key = BeautifiedFormat[format];
                return key ? defaultLocalizedStrings[key] || format : format;
            }, SandboxVisualHostServices.describeUnit = function(exponent) {
                var exponentLookup = exponent === -1 ? "Auto" : exponent.toString(), title = defaultLocalizedStrings["DisplayUnitSystem_E" + exponentLookup + "_Title"], format = exponent <= 0 ? "{0}" : defaultLocalizedStrings["DisplayUnitSystem_E" + exponentLookup + "_LabelFormat"];
                if (title || format) return {
                    title: title,
                    format: format
                };
            }, SandboxVisualHostServices.prototype.enableLoadMoreData = function(enable) {
                this.loadMoreDataEnabled = enable;
            }, SandboxVisualHostServices;
        }();
        visualSandbox.SandboxVisualHostServices = SandboxVisualHostServices, visualSandbox.createVisualHostServices = createVisualHostServices;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createNonSandboxVisualHostServices(host, element) {
            return new NonSandboxVisualHostServices(host, element);
        }
        var SelectionManager = powerbi.extensibility.SelectionManager, NonSandboxVisualHostServices = function() {
            function NonSandboxVisualHostServices(host, element) {
                this.loadMoreDataEnabled = !0, this.powerbiHost = host, this.visualHostTooltipService = this.powerbiHost.tooltips(), 
                this.tooltipService = new NonSandboxTooltipService({
                    hostServices: this
                }, element);
            }
            return NonSandboxVisualHostServices.prototype.onSelecting = function(selectingArgs) {
                selectingArgs.action = 0;
            }, NonSandboxVisualHostServices.prototype.onSelect = function(selectArgs) {
                this.powerbiHost && this.powerbiHost.onSelect(selectArgs);
            }, NonSandboxVisualHostServices.prototype.persistProperties = function(changes) {
                this.powerbiHost && this.powerbiHost.persistProperties(changes);
            }, NonSandboxVisualHostServices.prototype.tooltips = function() {
                return this.visualHostTooltipService;
            }, NonSandboxVisualHostServices.prototype.applyJsonFilter = function(filter, objectName, propertyName, action) {
                this.powerbiHost.applyJsonFilter(filter, objectName, propertyName, action);
            }, NonSandboxVisualHostServices.prototype.getJSONFilter = function(_filter) {
                return this.promiseFactory().reject();
            }, NonSandboxVisualHostServices.prototype.createSelectionIdBuilder = function() {
                return new powerbi.visuals.SelectionIdBuilder();
            }, NonSandboxVisualHostServices.prototype.createTooltipService = function() {
                return this.tooltipService;
            }, NonSandboxVisualHostServices.prototype.getLocalizedString = function(stringId) {
                return null;
            }, NonSandboxVisualHostServices.prototype.locale = function() {
                return navigator.language;
            }, NonSandboxVisualHostServices.prototype.loader = function() {
                var _this = this;
                return {
                    require: function() {
                        return _this.promiseFactory().reject();
                    }
                };
            }, NonSandboxVisualHostServices.prototype.loadMoreData = function() {}, NonSandboxVisualHostServices.prototype.fetchMoreData = function() {
                return !0;
            }, NonSandboxVisualHostServices.prototype.launchUrl = function(url) {}, NonSandboxVisualHostServices.prototype.destroy = function() {}, 
            NonSandboxVisualHostServices.prototype.refreshHostData = function() {}, NonSandboxVisualHostServices.prototype.onContextMenu = function() {}, 
            NonSandboxVisualHostServices.prototype.authenticationService = function() {
                return null;
            }, NonSandboxVisualHostServices.prototype.telemetry = function() {
                return null;
            }, NonSandboxVisualHostServices.prototype.localStorageService = function() {
                return null;
            }, NonSandboxVisualHostServices.prototype.eventService = function() {
                return null;
            }, NonSandboxVisualHostServices.prototype.requestFocus = function() {}, NonSandboxVisualHostServices.prototype.onDragStart = function() {}, 
            NonSandboxVisualHostServices.prototype.getVisualLinkProperties = function() {}, 
            NonSandboxVisualHostServices.prototype.canSelect = function() {
                return !1;
            }, NonSandboxVisualHostServices.prototype.canExpandCollapse = function() {
                return !1;
            }, NonSandboxVisualHostServices.prototype.setNodeExpansionState = function(_args) {}, 
            NonSandboxVisualHostServices.prototype.onCustomSort = function(args) {}, NonSandboxVisualHostServices.prototype.getViewMode = function() {
                return 0;
            }, NonSandboxVisualHostServices.prototype.getEditMode = function() {
                return 0;
            }, NonSandboxVisualHostServices.prototype.setWarnings = function(warnings) {}, NonSandboxVisualHostServices.prototype.setToolbar = function($toolbar) {}, 
            NonSandboxVisualHostServices.prototype.shouldRetainSelection = function() {
                return !1;
            }, NonSandboxVisualHostServices.prototype.geocoder = function() {
                return powerbi.visuals.services.createGeocoder();
            }, NonSandboxVisualHostServices.prototype.geolocation = function() {
                return powerbi.visuals.services.createGeolocation();
            }, NonSandboxVisualHostServices.prototype.promiseFactory = function() {
                return powerbi.createJQueryPromiseFactory();
            }, NonSandboxVisualHostServices.prototype.analyzeFilter = function(options) {}, 
            NonSandboxVisualHostServices.prototype.getIdentityDisplayNames = function(dentities) {}, 
            NonSandboxVisualHostServices.prototype.setIdentityDisplayNames = function(displayNamesIdentityPairs) {}, 
            NonSandboxVisualHostServices.prototype.getUIComponentFactory = function() {}, NonSandboxVisualHostServices.prototype.enableLoadMoreData = function(enable) {
                this.loadMoreDataEnabled = enable;
            }, NonSandboxVisualHostServices;
        }();
        visualSandbox.NonSandboxVisualHostServices = NonSandboxVisualHostServices;
        var NonSandboxTooltipService = function() {
            function NonSandboxTooltipService(options, element) {
                this.tooltipService = options.hostServices.tooltips(), this.element = element;
            }
            return NonSandboxTooltipService.prototype.enabled = function() {
                return this.tooltipService && this.tooltipService.enabled();
            }, NonSandboxTooltipService.prototype.show = function(options) {
                if (this.enabled()) {
                    var args = {
                        coordinates: this.fixCoordinates(options.coordinates),
                        dataItems: options.dataItems,
                        isTouchEvent: options.isTouchEvent,
                        identities: SelectionManager.getSelectorsByColumn(options.identities)
                    };
                    this.tooltipService.show(args);
                }
            }, NonSandboxTooltipService.prototype.move = function(options) {
                if (this.enabled()) {
                    var args = {
                        coordinates: this.fixCoordinates(options.coordinates),
                        dataItems: options.dataItems,
                        isTouchEvent: options.isTouchEvent,
                        identities: SelectionManager.getSelectorsByColumn(options.identities)
                    };
                    this.tooltipService.move(args);
                }
            }, NonSandboxTooltipService.prototype.hide = function(options) {
                this.enabled() && this.tooltipService.hide(options);
            }, NonSandboxTooltipService.prototype.fixCoordinates = function(coordinates) {
                var offset = this.element.offset();
                return coordinates[0] = coordinates[0] + offset.left, coordinates[1] = coordinates[1] + offset.top, 
                coordinates;
            }, NonSandboxTooltipService;
        }();
        visualSandbox.createNonSandboxVisualHostServices = createNonSandboxVisualHostServices;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        var DataViewUtils, DataViewObjectSerializer = powerbi.data.services.DataViewObjectSerializer;
        !function(DataViewUtils) {
            function serializeDataViewForCv(dataView, capabilities) {
                if (dataView) {
                    dataView = filterDataViewsType(dataView, capabilities);
                    var valueSource = getValueSource(dataView), valueGroups = getValueGroups(dataView), stringifiedDataView = JSON.stringify(dataView, replacer);
                    return {
                        valueSource: valueSource,
                        valueGroups: valueGroups,
                        strigifiedDataView: stringifiedDataView
                    };
                }
            }
            function serializeDataViewMetadata(dataView) {
                return dataView.metadata ? JSON.stringify(dataView.metadata, replacer) : null;
            }
            function deserializeDataView(serializedDataView) {
                var parsedDv = JSON.parse(serializedDataView.strigifiedDataView);
                return reconstructValuesGroups(parsedDv, serializedDataView), reconstructValuesSource(parsedDv, serializedDataView), 
                parsedDv;
            }
            function getValueGroups(dataView) {
                if (dataView && dataView.categorical && dataView.categorical.values && dataView.categorical.values.grouped) {
                    var valueGroups = dataView.categorical.values.grouped();
                    return JSON.stringify(valueGroups, replacer);
                }
                return null;
            }
            function getValueSource(dataView) {
                return dataView && dataView.categorical && dataView.categorical.values && dataView.categorical.values.source ? dataView.categorical.values.source : null;
            }
            function replacer(_key, value) {
                if (value) {
                    if (isDataViewScopeIdentity(value)) return DataViewObjectSerializer.serializeScopeIdentity(value);
                    if (value instanceof Array) {
                        var array = [];
                        return array.push.apply(array, value), array;
                    }
                    if ("object" == typeof value && !dontNeedToClone(value)) {
                        var newObject = {};
                        for (var objectKey in value) newObject[objectKey] = value[objectKey];
                        return newObject;
                    }
                }
                return value;
            }
            function filterDataViewsType(dataView, capabilities) {
                return capabilities.dataViewMappings ? (capabilities.dataViewMappings[0].categorical || (dataView.categorical = null), 
                capabilities.dataViewMappings[0].matrix || (dataView.matrix = null), capabilities.dataViewMappings[0].table || (dataView.table = null), 
                capabilities.dataViewMappings[0].tree || (dataView.tree = null), capabilities.dataViewMappings[0].single || (dataView.single = null), 
                dataView) : dataView;
            }
            function dontNeedToClone(value) {
                return value._proto_ === Object.prototype || value instanceof Date;
            }
            function isDataViewScopeIdentity(obj) {
                return !!(obj && obj.expr && obj.key && obj.kind);
            }
            function reconstructValuesSource(dataView, serializedDV) {
                serializedDV && serializedDV.valueSource && (dataView.categorical.values.source = serializedDV.valueSource);
            }
            function reconstructValuesGroups(dataView, serializedDV) {
                dataView && dataView.categorical && dataView.categorical.values && dataView.categorical.values[0] && (dataView.categorical.values.grouped = function() {
                    return JSON.parse(serializedDV.valueGroups);
                });
            }
            DataViewUtils.serializeDataViewForCv = serializeDataViewForCv, DataViewUtils.serializeDataViewMetadata = serializeDataViewMetadata, 
            DataViewUtils.deserializeDataView = deserializeDataView, DataViewUtils.replacer = replacer;
        }(DataViewUtils = visualSandbox.DataViewUtils || (visualSandbox.DataViewUtils = {}));
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));

var powerbi;

!function(powerbi) {
    var visualSandbox;
    !function(visualSandbox) {
        function createSandboxVisualHostAuthenticationService(messageProxy, promiseFactory, sandboxPromiseHandler) {
            return new SandboxVisualHostAuthenticationService(messageProxy, promiseFactory, sandboxPromiseHandler);
        }
        visualSandbox.createSandboxVisualHostAuthenticationService = createSandboxVisualHostAuthenticationService;
        var SandboxVisualHostAuthenticationService = function() {
            function SandboxVisualHostAuthenticationService(messageProxy, promiseFactory, sandboxPromiseHandler) {
                this.messageProxy = messageProxy, this.promiseFactory = promiseFactory, this.sandboxPromiseHandler = sandboxPromiseHandler;
            }
            return SandboxVisualHostAuthenticationService.prototype.getAADToken = function() {
                var promise = this.sandboxPromiseHandler.CreatePromise(this.promiseFactory.defer()), message = {
                    eventName: "getAADToken",
                    arguments: [ promise.id ]
                };
                return this.messageProxy.postMessage(message), promise.deferred.promise;
            }, SandboxVisualHostAuthenticationService.prototype.getWACToken = function(resourceSiteUrl, resourceEtag) {
                var promise2 = this.sandboxPromiseHandler.CreatePromise2(this.promiseFactory.defer()), proxyArgs = {
                    resourceEtag: resourceEtag,
                    resourceSiteUrl: resourceSiteUrl,
                    promiseId: promise2.id
                }, message = {
                    eventName: "getWACToken",
                    arguments: [ proxyArgs ]
                };
                return this.messageProxy.postMessage(message), promise2.deferred.promise;
            }, SandboxVisualHostAuthenticationService;
        }();
        visualSandbox.SandboxVisualHostAuthenticationService = SandboxVisualHostAuthenticationService;
    }(visualSandbox = powerbi.visualSandbox || (powerbi.visualSandbox = {}));
}(powerbi || (powerbi = {}));